@@ -22,16 +22,15 @@ class FilterSelect:
         """
         Conduct F-test of the interaction between treatment and one feature.
 
-        Parameters
-        ----------
-        data (pd.Dataframe): DataFrame containing outcome, features, and experiment group
-        treatment_indicator (string): the column name for binary indicator of treatment (value 1) or control (value 0) 
-        feature_name (string): feature name, as one column in the data DataFrame
-        y_name (string): name of the outcome variable
+        Args:
+            data (pd.Dataframe): DataFrame containing outcome, features, and experiment group
+            treatment_indicator (string): the column name for binary indicator of treatment (value 1) or control (value 0)
+            feature_name (string): feature name, as one column in the data DataFrame
+            y_name (string): name of the outcome variable
 
-        Returns
-        ----------
-        (pd.DataFrame): a data frame containing the feature importance statistics
+        Returns:
+            F_test_result : pd.DataFrame
+                a data frame containing the feature importance statistics
         """
         Y = data[y_name]
         X = data[[treatment_indicator, feature_name]]
@@ -45,9 +44,9 @@ class FilterSelect:
         F_test_result = pd.DataFrame({
             'feature': feature_name, # for the interaction, not the main effect
             'method': 'F-statistic',
-            'score': F_test.fvalue[0][0], 
-            'p_value': F_test.pvalue, 
-            'misc': 'df_num: {}, df_denom: {}'.format(F_test.df_num, F_test.df_denom), 
+            'score': F_test.fvalue[0][0],
+            'p_value': F_test.pvalue,
+            'misc': 'df_num: {}, df_denom: {}'.format(F_test.df_num, F_test.df_denom),
         }, index=[0]).reset_index(drop=True)
 
         return F_test_result
@@ -57,19 +56,18 @@ class FilterSelect:
         """
         Rank features based on the F-statistics of the interaction.
 
-        Parameters
-        ----------
-        data (pd.Dataframe): DataFrame containing outcome, features, and experiment group
-        treatment_indicator (string): the column name for binary indicator of treatment (value 1) or control (value 0) 
-        features (list of string): list of feature names, that are columns in the data DataFrame
-        y_name (string): name of the outcome variable
+        Args:
+            data (pd.Dataframe): DataFrame containing outcome, features, and experiment group
+            treatment_indicator (string): the column name for binary indicator of treatment (value 1) or control (value 0)
+            features (list of string): list of feature names, that are columns in the data DataFrame
+            y_name (string): name of the outcome variable
 
-        Returns
-        ----------
-        (pd.DataFrame): a data frame containing the feature importance statistics
+        Returns:
+            all_result : pd.DataFrame
+                a data frame containing the feature importance statistics
         """
         all_result = pd.DataFrame()
-        for x_name_i in features: 
+        for x_name_i in features:
             one_result = self._filter_F_one_feature(data=data,
                 treatment_indicator=treatment_indicator, feature_name=x_name_i, y_name=y_name
             )
@@ -82,23 +80,22 @@ class FilterSelect:
 
 
     @staticmethod
-    def _filter_LR_one_feature(data, treatment_indicator, feature_name, y_name, disp=True):  
+    def _filter_LR_one_feature(data, treatment_indicator, feature_name, y_name, disp=True):
         """
         Conduct LR (Likelihood Ratio) test of the interaction between treatment and one feature.
 
-        Parameters
-        ----------
-        data (pd.Dataframe): DataFrame containing outcome, features, and experiment group
-        treatment_indicator (string): the column name for binary indicator of treatment (value 1) or control (value 0) 
-        feature_name (string): feature name, as one column in the data DataFrame
-        y_name (string): name of the outcome variable
+        Args:
+            data (pd.Dataframe): DataFrame containing outcome, features, and experiment group
+            treatment_indicator (string): the column name for binary indicator of treatment (value 1) or control (value 0)
+            feature_name (string): feature name, as one column in the data DataFrame
+            y_name (string): name of the outcome variable
 
-        Returns
-        ----------
-        (pd.DataFrame): a data frame containing the feature importance statistics
+        Returns:
+            LR_test_result : pd.DataFrame
+                a data frame containing the feature importance statistics
         """
         Y = data[y_name]
-        
+
         # Restricted model
         X_r = data[[treatment_indicator, feature_name]]
         X_r = sm.add_constant(X_r)
@@ -118,9 +115,9 @@ class FilterSelect:
         LR_test_result = pd.DataFrame({
             'feature': feature_name, # for the interaction, not the main effect
             'method': 'LRT-statistic',
-            'score': LR_stat, 
+            'score': LR_stat,
             'p_value': LR_pvalue,
-            'misc': 'df: {}'.format(LR_df), 
+            'misc': 'df: {}'.format(LR_df),
         }, index=[0]).reset_index(drop=True)
 
         return LR_test_result
@@ -130,20 +127,19 @@ class FilterSelect:
         """
         Rank features based on the LRT-statistics of the interaction.
 
-        Parameters
-        ----------
-        data (pd.Dataframe): DataFrame containing outcome, features, and experiment group
-        treatment_indicator (string): the column name for binary indicator of treatment (value 1) or control (value 0) 
-        feature_name (string): feature name, as one column in the data DataFrame
-        y_name (string): name of the outcome variable
+        Args:
+            data (pd.Dataframe): DataFrame containing outcome, features, and experiment group
+            treatment_indicator (string): the column name for binary indicator of treatment (value 1) or control (value 0)
+            feature_name (string): feature name, as one column in the data DataFrame
+            y_name (string): name of the outcome variable
 
-        Returns
-        ----------
-        (pd.DataFrame): a data frame containing the feature importance statistics
+        Returns:
+            all_result : pd.DataFrame
+                a data frame containing the feature importance statistics
         """
         all_result = pd.DataFrame()
-        for x_name_i in features: 
-            one_result = self._filter_LR_one_feature(data=data, 
+        for x_name_i in features:
+            one_result = self._filter_LR_one_feature(data=data,
                 treatment_indicator=treatment_indicator, feature_name=x_name_i, y_name=y_name, disp=disp
             )
             all_result = pd.concat([all_result, one_result])
@@ -154,10 +150,10 @@ class FilterSelect:
         return all_result
 
 
-    # Get node summary - a function 
+    # Get node summary - a function
     @staticmethod
     def _GetNodeSummary(data,
-                        experiment_group_column='treatment_group_key', 
+                        experiment_group_column='treatment_group_key',
                         y_name='conversion', smooth=True):
         """
         To count the conversions and get the probabilities by treatment groups. This function comes from the uplift tree algorithm, that is used for tree node split evaluation.
@@ -165,7 +161,7 @@ class FilterSelect:
         Parameters
         ----------
         data : DataFrame
-            The DataFrame that contains all the data (in the current "node").  
+            The DataFrame that contains all the data (in the current "node").
         experiment_group_column : str
             Treatment indicator column name.
         y_name : str
@@ -177,27 +173,27 @@ class FilterSelect:
         Returns
         -------
         results : dict
-            Counts of conversions by treatment groups, of the form: 
+            Counts of conversions by treatment groups, of the form:
             {'control': {0: 10, 1: 8}, 'treatment1': {0: 5, 1: 15}}
         nodeSummary: dict
-            Probability of conversion and group size by treatment groups, of 
+            Probability of conversion and group size by treatment groups, of
             the form:
             {'control': [0.490, 500], 'treatment1': [0.584, 500]}
         """
 
         # Note: results and nodeSummary are both dict with treatment_group_key
-        # as the key.  So we can compute the treatment effect and/or 
+        # as the key.  So we can compute the treatment effect and/or
         # divergence easily.
 
         # Counts of conversions by treatment group
         results_series = data.groupby([experiment_group_column, y_name]).size()
-        
+
         treatment_group_keys = results_series.index.levels[0].tolist()
         y_name_keys = results_series.index.levels[1].tolist()
 
         results = {}
-        for ti in treatment_group_keys: 
-            results.update({ti: {}}) 
+        for ti in treatment_group_keys:
+            results.update({ti: {}})
             for ci in y_name_keys:
                 if smooth:
                     results[ti].update({ci: results_series[ti, ci]
@@ -208,14 +204,14 @@ class FilterSelect:
 
         # Probability of conversion and group size by treatment group
         nodeSummary = {}
-        for treatment_group_key in results: 
+        for treatment_group_key in results:
             n_1 = results[treatment_group_key].get(1, 0)
-            n_total = (results[treatment_group_key].get(1, 0) 
+            n_total = (results[treatment_group_key].get(1, 0)
                        + results[treatment_group_key].get(0, 0))
             y_mean = 1.0 * n_1 / n_total
             nodeSummary[treatment_group_key] = [y_mean, n_total]
-        
-        return results, nodeSummary 
+
+        return results, nodeSummary
 
     # Divergence-related functions, from upliftpy
     @staticmethod
@@ -223,10 +219,9 @@ class FilterSelect:
         """
         Calculate KL Divergence for binary classification.
 
-        Parameters
-        ----------
-        pk (float): Probability of class 1 in treatment group
-        qk (float): Probability of class 1 in control group
+        Args:
+            pk (float): Probability of class 1 in treatment group
+            qk (float): Probability of class 1 in control group
         """
         if qk < 0.1**6:
             qk = 0.1**6
@@ -240,10 +235,9 @@ class FilterSelect:
         Calculate the multi-treatment unconditional D (one node)
         with KL Divergence as split Evaluation function.
 
-        Parameters
-        ----------
-        nodeSummary (dict): a dictionary containing the statistics for a tree node sample
-        control_group (string, optional, default='control'): the name for control group 
+        Args:
+            nodeSummary (dict): a dictionary containing the statistics for a tree node sample
+            control_group (string, optional, default='control'): the name for control group
 
         Notes
         -----
@@ -264,10 +258,9 @@ class FilterSelect:
         Calculate the multi-treatment unconditional D (one node)
         with Euclidean Distance as split Evaluation function.
 
-        Parameters
-        ----------
-        nodeSummary (dict): a dictionary containing the statistics for a tree node sample
-        control_group (string, optional, default='control'): the name for control group        
+        Args:
+            nodeSummary (dict): a dictionary containing the statistics for a tree node sample
+            control_group (string, optional, default='control'): the name for control group
         """
         if control_group not in nodeSummary:
             return 0
@@ -284,11 +277,9 @@ class FilterSelect:
         Calculate the multi-treatment unconditional D (one node)
         with Chi-Square as split Evaluation function.
 
-        Parameters
-        ----------
-        nodeSummary (dict): a dictionary containing the statistics for a tree node sample
-        control_group (string, optional, default='control'): the name for control group 
-
+        Args:
+            nodeSummary (dict): a dictionary containing the statistics for a tree node sample
+            control_group (string, optional, default='control'): the name for control group
         """
         if control_group not in nodeSummary:
             return 0
@@ -297,37 +288,37 @@ class FilterSelect:
         for treatment_group in nodeSummary:
             if treatment_group != control_group:
                 d_res += (
-                    (nodeSummary[treatment_group][0] - pc)**2 / max(0.1**6, pc) 
+                    (nodeSummary[treatment_group][0] - pc)**2 / max(0.1**6, pc)
                     + (nodeSummary[treatment_group][0] - pc)**2 / max(0.1**6, 1-pc)
                 )
         return d_res
 
 
-    def _filter_D_one_feature(self, data, feature_name, y_name, 
+    def _filter_D_one_feature(self, data, feature_name, y_name,
                               n_bins=10, method='KL', control_group='control',
                               experiment_group_column='treatment_group_key',
                               null_impute=None):
         """
         Calculate the chosen divergence measure for one feature.
 
-        Parameters
-        ----------
-        data (pd.Dataframe): DataFrame containing outcome, features, and experiment group
-        treatment_indicator (string): the column name for binary indicator of treatment (value 1) or control (value 0) 
-        feature_name (string): feature name, as one column in the data DataFrame
-        y_name (string): name of the outcome variable
-        method (string, optional, default = 'KL'): taking one of the following values {'F', 'LR', 'KL', 'ED', 'Chi'}
-                The feature selection method to be used to rank the features.  
-                'F' for F-test 
-                'LR' for likelihood ratio test
-                'KL', 'ED', 'Chi' for bin-based uplift filter methods, KL divergence, Euclidean distance, Chi-Square respectively
-        experiment_group_column (string, optional, default = 'treatment_group_key'): the experiment column name in the DataFrame, which contains the treatment and control assignment label
-        control_group (string, optional, default = 'control'): name for control group, value in the experiment group column
-        n_bins (int, optional, default = 10): number of bins to be used for bin-based uplift filter methods
-        null_impute (str, optional, default=None): impute np.nan present in the data taking on of the following strategy values {'mean', 'median', 'most_frequent', None}. If Value is None and null is present then exception will be raised
-        Returns
-        ----------
-        (pd.DataFrame): a data frame containing the feature importance statistics
+        Args:
+            data (pd.Dataframe): DataFrame containing outcome, features, and experiment group
+            treatment_indicator (string): the column name for binary indicator of treatment (value 1) or control (value 0)
+            feature_name (string): feature name, as one column in the data DataFrame
+            y_name (string): name of the outcome variable
+            method (string, optional, default = 'KL'): taking one of the following values {'F', 'LR', 'KL', 'ED', 'Chi'}
+                    The feature selection method to be used to rank the features.
+                    'F' for F-test
+                    'LR' for likelihood ratio test
+                    'KL', 'ED', 'Chi' for bin-based uplift filter methods, KL divergence, Euclidean distance, Chi-Square respectively
+            experiment_group_column (string, optional, default = 'treatment_group_key'): the experiment column name in the DataFrame, which contains the treatment and control assignment label
+            control_group (string, optional, default = 'control'): name for control group, value in the experiment group column
+            n_bins (int, optional, default = 10): number of bins to be used for bin-based uplift filter methods
+            null_impute (str, optional, default=None): impute np.nan present in the data taking on of the following strategy values {'mean', 'median', 'most_frequent', None}. If Value is None and null is present then exception will be raised
+
+        Returns:
+            D_result : pd.DataFrame
+                a data frame containing the feature importance statistics
         """
         # [TODO] Application to categorical features
 
@@ -347,17 +338,17 @@ class FilterSelect:
             raise Exception("Null value(s) present in column '{}'. Please impute the null value or use null_impute parameter provided!!!".format(feature_name))
 
         # drop duplicate edges in pq.cut result to avoid issues
-        x_bin = pd.qcut(data[feature_name].values, n_bins, labels=False, 
+        x_bin = pd.qcut(data[feature_name].values, n_bins, labels=False,
                         duplicates='drop')
 
         d_children = 0
 
         for i_bin in range(np.nanmax(x_bin).astype(int) + 1): # range(n_bins):
             nodeSummary = self._GetNodeSummary(
-                data=data.loc[x_bin == i_bin], 
+                data=data.loc[x_bin == i_bin],
                 experiment_group_column=experiment_group_column, y_name=y_name
             )[1]
-            nodeScore = evaluationFunction(nodeSummary, 
+            nodeScore = evaluationFunction(nodeSummary,
                                            control_group=control_group)
             nodeSize = sum([x[1] for x in list(nodeSummary.values())])
             d_children += nodeScore * nodeSize / totalSize
@@ -365,56 +356,55 @@ class FilterSelect:
         parentNodeSummary = self._GetNodeSummary(
             data=data, experiment_group_column=experiment_group_column, y_name=y_name
         )[1]
-        d_parent = evaluationFunction(parentNodeSummary, 
+        d_parent = evaluationFunction(parentNodeSummary,
                                       control_group=control_group)
-            
+
         d_res = d_children - d_parent
-        
+
         D_result = pd.DataFrame({
-            'feature': feature_name, 
+            'feature': feature_name,
             'method': method,
-            'score': d_res, 
+            'score': d_res,
             'p_value': None,
             'misc': 'number_of_bins: {}'.format(min(n_bins, np.nanmax(x_bin).astype(int) + 1)),# format(n_bins),
         }, index=[0]).reset_index(drop=True)
 
         return(D_result)
 
-    def filter_D(self, data, features, y_name, 
+    def filter_D(self, data, features, y_name,
                  n_bins=10, method='KL', control_group='control',
                  experiment_group_column='treatment_group_key',
                  null_impute=None):
         """
         Rank features based on the chosen divergence measure.
 
-        Parameters
-        ----------
-        data (pd.Dataframe): DataFrame containing outcome, features, and experiment group
-        treatment_indicator (string): the column name for binary indicator of treatment (value 1) or control (value 0) 
-        features (list of string): list of feature names, that are columns in the data DataFrame
-        y_name (string): name of the outcome variable
-        method (string, optional, default = 'KL'): taking one of the following values {'F', 'LR', 'KL', 'ED', 'Chi'}
-                The feature selection method to be used to rank the features.  
-                'F' for F-test 
-                'LR' for likelihood ratio test
-                'KL', 'ED', 'Chi' for bin-based uplift filter methods, KL divergence, Euclidean distance, Chi-Square respectively
-        experiment_group_column (string, optional, default = 'treatment_group_key'): the experiment column name in the DataFrame, which contains the treatment and control assignment label
-        control_group (string, optional, default = 'control'): name for control group, value in the experiment group column
-        n_bins (int, optional, default = 10): number of bins to be used for bin-based uplift filter methods
-        null_impute (str, optional, default=None): impute np.nan present in the data taking on of the following strategy values {'mean', 'median', 'most_frequent', None}. If Value is None and null is present then exception will be raised
+        Args:
+            data (pd.Dataframe): DataFrame containing outcome, features, and experiment group
+            treatment_indicator (string): the column name for binary indicator of treatment (value 1) or control (value 0)
+            features (list of string): list of feature names, that are columns in the data DataFrame
+            y_name (string): name of the outcome variable
+            method (string, optional, default = 'KL'): taking one of the following values {'F', 'LR', 'KL', 'ED', 'Chi'}
+                    The feature selection method to be used to rank the features.
+                    'F' for F-test
+                    'LR' for likelihood ratio test
+                    'KL', 'ED', 'Chi' for bin-based uplift filter methods, KL divergence, Euclidean distance, Chi-Square respectively
+            experiment_group_column (string, optional, default = 'treatment_group_key'): the experiment column name in the DataFrame, which contains the treatment and control assignment label
+            control_group (string, optional, default = 'control'): name for control group, value in the experiment group column
+            n_bins (int, optional, default = 10): number of bins to be used for bin-based uplift filter methods
+            null_impute (str, optional, default=None): impute np.nan present in the data taking on of the following strategy values {'mean', 'median', 'most_frequent', None}. If Value is None and null is present then exception will be raised
 
-        Returns
-        ----------
-        (pd.DataFrame): a data frame containing the feature importance statistics
+        Returns:
+            all_result : pd.DataFrame
+                a data frame containing the feature importance statistics
         """
-        
+
         all_result = pd.DataFrame()
 
-        for x_name_i in features: 
+        for x_name_i in features:
             one_result = self._filter_D_one_feature(
                 data=data, feature_name=x_name_i, y_name=y_name,
                 n_bins=n_bins, method=method, control_group=control_group,
-                experiment_group_column=experiment_group_column, 
+                experiment_group_column=experiment_group_column,
                 null_impute=null_impute
             )
             all_result = pd.concat([all_result, one_result])
@@ -424,9 +414,9 @@ class FilterSelect:
 
         return all_result
 
-    def get_importance(self, data, features, y_name, method, 
+    def get_importance(self, data, features, y_name, method,
                       experiment_group_column='treatment_group_key',
-                      control_group = 'control', 
+                      control_group = 'control',
                       treatment_group = 'treatment',
                       n_bins=5,
                       null_impute=None
@@ -434,14 +424,13 @@ class FilterSelect:
         """
         Rank features based on the chosen statistic of the interaction.
 
-        Parameters
-        ----------
+        Args:
             data (pd.Dataframe): DataFrame containing outcome, features, and experiment group
             features (list of string): list of feature names, that are columns in the data DataFrame
             y_name (string): name of the outcome variable
             method (string, optional, default = 'KL'): taking one of the following values {'F', 'LR', 'KL', 'ED', 'Chi'}
-                The feature selection method to be used to rank the features.  
-                'F' for F-test 
+                The feature selection method to be used to rank the features.
+                'F' for F-test
                 'LR' for likelihood ratio test
                 'KL', 'ED', 'Chi' for bin-based uplift filter methods, KL divergence, Euclidean distance, Chi-Square respectively
             experiment_group_column (string): the experiment column name in the DataFrame, which contains the treatment and control assignment label
@@ -449,10 +438,10 @@ class FilterSelect:
             treatment_group (string): name for treatment group, value in the experiment group column
             n_bins (int, optional): number of bins to be used for bin-based uplift filter methods
             null_impute (str, optional, default=None): impute np.nan present in the data taking on of the following strategy values {'mean', 'median', 'most_frequent', None}. If value is None and null is present then exception will be raised
-        
-        Returns
-        ----------
-            (pd.DataFrame): a data frame with following columns: ['method', 'feature', 'rank', 'score', 'p_value', 'misc']
+
+        Returns:
+            all_result : pd.DataFrame
+                a data frame with following columns: ['method', 'feature', 'rank', 'score', 'p_value', 'misc']
         """
         
         if method == 'F':
