@@ -1,5 +1,4 @@
-from .models import UpliftTreeClassifier, DecisionTree
-from .models import UpliftRandomForestClassifier
+from .uplift import DecisionTree, UpliftTreeClassifier, UpliftRandomForestClassifier
 from  .causaltree import CausalMSE, CausalTreeRegressor
 from .plot import uplift_tree_string, uplift_tree_plot
 from .utils import cat_group, cat_transform, cv_fold_index, cat_continuous, kpi_transform
